﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml.Serialization;

namespace SampleApp
{
    public partial class Form1 : Form
    {
        private Function func = new Function();
        private Function.RecordData recordData = new Function.RecordData();

        public Form1()
        {
            //初期化処理
            InitializeComponent();

            //DataSelectComboBox初期化
            DataSelectComboBox.SelectedIndex = 0;

            //PhysicalActiveComboBox初期化
            PhysicalActiveComboBox.SelectedIndex = 1;

            //記録情報初期化
            UpdateInfo(func);

            //記録データ初期化
            UpdateRecordData(func, recordData);
        }

        /// <summary>
        /// 記録情報更新
        /// </summary>
        private void UpdateInfo(Function func)
        {
            //モジュール情報更新
            UpdateModuleInfo(func);
            //本体記録情報更新
            UpdateRecordInfo(func);
            //プリンタ記録情報更新
            UpdatePrinterInfo(func);
            //SSD記録情報更新
            UpdateSSDInfo(func);
            //メモリ記録情報更新
            UpdateMemoryInfo(func);
        }

        /// <summary>
        /// モジュール情報更新
        /// </summary>
        private void UpdateModuleInfo(Function func)
        {
            //データグリッドクリア
            ModuleInfoGrid.Rows.Clear();

            if (func.RecordInfo != null)
            {
                //モジュール名取得
                for (int sc = 1; sc <= Function.RA3100Const.MaxSlot; sc++)
                {
                    string modulename = string.Empty;
                    foreach (Slot slot in func.RecordInfo.CHInfo.Slots)
                    {
                        if (slot.No == sc)
                        {
                            modulename = slot.Module;
                        }
                    }
                    ModuleInfoGrid.Rows.Add(sc.ToString(), modulename);
                }
            }
        }

        /// <summary>
        /// 本体記録情報更新
        /// </summary>
        private void UpdateRecordInfo(Function func)
        {
            if (func.RecordInfo != null)
            {
                //本体情報
                SerialNumber.Text = func.RecordInfo.RecordInfo.SerialNumber;
                RA3100Name.Text = func.RecordInfo.RecordInfo.PCName;
                Version.Text = func.RecordInfo.RecordInfo.Version;

                //記録
                RecordName.Text = func.RecordInfo.RecordInfo.RecordTitle;
                RecordDate.Text = func.RecordInfo.RecordInfo.RecordTime.Value;

                //記録モードがウィンドウ記録の場合、記録時刻が終了時刻(RecordTime.When.End)になる
                if (func.RecordInfo.RecordInfo.RecordTime.When == RecordTimeWhen.Start)
                {
                    RecordDateLabel.Text = "開始日時";
                }
                else
                {
                    RecordDateLabel.Text = "終了日時";
                }
            }
            else
            {
                //本体記録情報クリア
                SerialNumber.Text = string.Empty;
                RA3100Name.Text = string.Empty;
                Version.Text = string.Empty;
                RecordName.Text = string.Empty;
                RecordDate.Text = string.Empty;
            }
        }

        /// <summary>
        /// プリンタ記録情報更新
        /// </summary>
        private void UpdatePrinterInfo(Function func)
        {
            if ((func.RecordInfo != null) && (func.RecordInfo.PrinterInfo != null))
            {
                //プリンタ記録情報
                PrinterActive.Text = "ON";
                PrinterSamplingCycle.Text = GetXmlEnumAttribute(func.RecordInfo.PrinterInfo.SamplingCycle);
                PrinterSampleCount.Text = func.RecordInfo.PrinterInfo.SampleCount.ToString();
            }
            else
            {
                //プリンタ記録情報クリア
                PrinterActive.Text = "OFF";
                PrinterSamplingCycle.Text = string.Empty;
                PrinterSampleCount.Text = string.Empty;
            }
        }

        /// <summary>
        /// SSD記録情報更新
        /// </summary>
        private void UpdateSSDInfo(Function func)
        {
            if ((func.RecordInfo != null) && (func.RecordInfo.SSDInfo != null))
            {
                //SSD記録情報
                SSDActive.Text = "ON";
                SSDSamplingCycle.Text = GetXmlEnumAttribute(func.RecordInfo.SSDInfo.SamplingCycle);
                SSDSampleCount.Text = func.RecordInfo.SSDInfo.SampleCount.ToString();
                SSDDataFormat.Text = GetXmlEnumAttribute(func.RecordInfo.SSDInfo.DataFormat);
            }
            else
            {
                //SSD記録情報クリア
                SSDActive.Text = "OFF";
                SSDSamplingCycle.Text = string.Empty;
                SSDSampleCount.Text = string.Empty;
                SSDDataFormat.Text = string.Empty;
            }
        }

        /// <summary>
        /// メモリ記録情報更新
        /// </summary>
        private void UpdateMemoryInfo(Function func)
        {
            if ((func.RecordInfo != null) && (func.RecordInfo.MemoryInfo != null))
            {
                //メモリ記録情報
                MemoryActive.Text = "ON";
                MemorySamplingCycle.Text = GetXmlEnumAttribute(func.RecordInfo.MemoryInfo.SamplingCycle);
                MemoryBlockSampleCount.Text = GetXmlEnumAttribute(func.RecordInfo.MemoryInfo.BlockSamplePoints);
                MemoryTriggerPoint.Text = func.RecordInfo.MemoryInfo.PreTriggerPosition.Value.ToString() + func.RecordInfo.MemoryInfo.PreTriggerPosition.Unit;
                MemoryEndlessMode.Text = GetXmlEnumAttribute(func.RecordInfo.MemoryInfo.EndlessMode);

                //記録したブロック数
                if (func.RecordInfo.MemoryInfo.RecordedBlocks != null)
                {
                    MemoryBlockCount.Text = func.RecordInfo.MemoryInfo.RecordedBlocks.Length.ToString();
                }
                else
                {
                    MemoryBlockCount.Text = "0";
                }
            }
            else
            {
                //メモリ記録情報クリア
                MemoryActive.Text = "OFF";
                MemorySamplingCycle.Text = string.Empty;
                MemoryBlockSampleCount.Text = string.Empty;
                MemoryTriggerPoint.Text = string.Empty;
                MemoryEndlessMode.Text = string.Empty;
                MemoryBlockCount.Text = string.Empty;
            }
        }

        /// <summary>
        /// 記録データ情報更新
        /// </summary>
        private void UpdateRecordData(Function func, Function.RecordData recordData)
        {
            if (func.RecordInfo != null)
            {
                //各記録の1サンプルのデータサイズ
                switch (DataSelectComboBox.SelectedItem.ToString())
                {
                    case "プリンタ":
                        //プリンタ記録情報
                        if (func.RecordInfo.PrinterInfo != null)
                        {
                            //プリンタ記録の1サンプルのデータサイズ
                            OneSampleSize.Text = func.RecordInfo.PrinterInfo.OneSampleSize.ToString() + "Byte";
                        }
                        MemTrgPointLabel.Enabled = false;
                        MemTrgPoint.Text = string.Empty;
                        break;

                    case "SSD":
                        //SSD記録情報
                        if (func.RecordInfo.SSDInfo != null)
                        {
                            //SSD記録の1サンプルのデータサイズ
                            OneSampleSize.Text = func.RecordInfo.SSDInfo.OneSampleSize.ToString() + "Byte";
                        }
                        MemTrgPointLabel.Enabled = false;
                        MemTrgPoint.Text = string.Empty;
                        break;

                    case "メモリ":
                        //メモリ記録情報
                        if (func.RecordInfo.MemoryInfo != null)
                        {
                            //メモリ記録の1サンプルのデータサイズ
                            OneSampleSize.Text = func.RecordInfo.MemoryInfo.OneSampleSize.ToString() + "Byte";
                        }

                        //メモリ記録にはバイナリデータにステータス情報(トリガ、マーク)が付加されていないため、メモリブロック毎にメモリトリガ位置情報が用意される
                        MemTrgPointLabel.Enabled = true;
                        MemTrgPoint.Text = recordData.MemoryTriggerPoint.ToString();
                        break;
                }

                //読み込んだファイル名
                FileName.Text = string.Empty;
                foreach (string readFileName in recordData.ReadFileName)
                {
                    FileName.Text += readFileName + " ";
                }

                //1ファイルのサンプル数
                FileSampleCount.Text = recordData.FileSampleCount.ToString();
            }
            else
            {
                //記録データ情報クリア
                FileName.Text = string.Empty;
                OneSampleSize.Text = string.Empty;
                FileSampleCount.Text = string.Empty;
                MemTrgPointLabel.Enabled = false;
                MemTrgPoint.Text = string.Empty;
            }

            //記録データグリッド更新
            RecordDataGrid.Rows.Clear();
            if (recordData.Value.Count != 0)
            {
                //記録データヘッダーテキスト更新
                RecordDataGrid.ColumnCount = recordData.HeaderText.Count;
                for (int c = 0; c < RecordDataGrid.ColumnCount; c++)
                {
                    RecordDataGrid.Columns[c].HeaderText = recordData.HeaderText[c];
                }

                //記録データ値更新
                for (int r = 0; r < recordData.Value.Count; r++)
                {
                    RecordDataGrid.Rows.Add();
                    for (int c = 0; c < RecordDataGrid.ColumnCount; c++)
                    {
                        RecordDataGrid.Rows[r].Cells[c].Value = recordData.Value[r][c];
                    }
                }
            }
        }

        /// <summary>
        /// Enumの選択文字列を取得
        /// </summary>
        private string GetXmlEnumAttribute(Enum enumObj)
        {
            FieldInfo fieldInfo = enumObj.GetType().GetField(enumObj.ToString());
            object[] attribArray = fieldInfo.GetCustomAttributes(false);
            if (attribArray.Length == 0)
            {
                return enumObj.ToString();
            }
            else
            {
                XmlEnumAttribute attrib = attribArray[0] as XmlEnumAttribute;
                string name = attrib.Name;
                return name;
            }
        }

        /// <summary>
        /// 記録フォルダ選択ボタンのClickイベント
        /// </summary>
        private void RecordFolderSelectButton_Click(object sender, EventArgs e)
        {
            //記録フォルダ選択ダイアログを表示
            FolderBrowserDialog folder = new FolderBrowserDialog();
            if (folder.ShowDialog() == DialogResult.OK)
            {
                RecordFolderPath.Text = folder.SelectedPath;
            }

        }

        /// <summary>
        /// XML読み込みボタンのClickイベント
        /// </summary>
        private void RecordReadButton_Click(object sender, EventArgs e)
        {
            //XMLファイルから記録情報を取得
            if (!func.GetRecordInfo(RecordFolderPath.Text))
            {
                MessageBox.Show("XMLファイルの読み込みに失敗しました");
            }

            //記録情報更新
            UpdateInfo(func);
        }

        /// <summary>
        /// 記録データ読み込みボタンのClickイベント
        /// </summary>
        private void RecordDataReadButton_Click(object sender, EventArgs e)
        {
            bool ret = false;
            try
            {
                //読み込み条件を設定
                long readSamplePoint = long.Parse(ReadSamplePoint.Text);
                long readSampleCount = long.Parse(ReadSampleCount.Text);
                bool physicalActive = Convert.ToBoolean(PhysicalActiveComboBox.SelectedIndex);
                int readBlockNumber = int.Parse(ReadBlockNumber.Text);

                //設定制限(開始サンプル点)
                if (readSamplePoint < 1)
                {
                    readSamplePoint = 1;
                    ReadSamplePoint.Text = readSamplePoint.ToString();
                }

                //設定制限(サンプル数)
                if (readSampleCount < 1)
                {
                    readSampleCount = 1;
                    ReadSampleCount.Text = readSampleCount.ToString();
                }
                if (readSampleCount > 5000)
                {
                    readSampleCount = 5000;
                    ReadSampleCount.Text = readSampleCount.ToString();
                }

                //読み込む記録データを選択
                switch (DataSelectComboBox.SelectedItem.ToString())
                {
                    case "プリンタ":
                        //プリンタ記録のBINファイルから記録データを取得する
                        ret = func.GetPrinterRecordData(RecordFolderPath.Text, readSamplePoint, readSampleCount, physicalActive, ref recordData);
                        break;

                    case "SSD":
                        //SSD記録のBINファイルから記録データを取得する
                        ret = func.GetSSDRecordData(RecordFolderPath.Text, readSamplePoint, readSampleCount, physicalActive, ref recordData);
                        break;

                    case "メモリ":
                        //メモリ記録のBINファイルから記録データを取得する
                        ret = func.GetMemoryRecordData(RecordFolderPath.Text, readSamplePoint, readSampleCount, physicalActive, readBlockNumber, ref recordData);
                        break;
                }
            }
            catch (Exception)
            {
                ret = false;
            }

            if (!ret)
            {
                MessageBox.Show("記録データの読み込みに失敗しました");
            }

            //記録データ情報更新
            UpdateRecordData(func, recordData);
        }

        /// <summary>
        /// データ選択コンボボックスのSelectedIndexChangedイベント
        /// </summary>
        private void DataSelectComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            //読み込む記録データを選択
            if (DataSelectComboBox.SelectedItem.ToString() == "メモリ")
            {
                //メモリ記録の場合ブロックNo.設定を有効
                ReadBlockNumberLabel.Enabled = true;
                ReadBlockNumber.Enabled = true;
            }
            else
            {
                //プリンタ記録、SSD記録の場合ブロックNo.設定を無効
                ReadBlockNumberLabel.Enabled = false;
                ReadBlockNumber.Enabled = false;
            }
        }

        /// <summary>
        /// 記録フォルダTextBoxのDragEnterイベント
        /// </summary>
        private void RecordFolderPath_DragEnter(object sender, DragEventArgs e)
        {
            e.Effect = DragDropEffects.All;
        }

        /// <summary>
        /// 記録フォルダTextBoxのDragDropイベント
        /// </summary>
        private void RecordFolderPath_DragDrop(object sender, DragEventArgs e)
        {
            //Drag&Dropされたファイル、フォルダ名のパスを設定する
            if (!e.Data.GetDataPresent(DataFormats.FileDrop)) return;
            string[] dragFilePathArr = (string[])e.Data.GetData(DataFormats.FileDrop, false);
            RecordFolderPath.Text = dragFilePathArr[0];
        }
    }
}
